@@ -1,8 +1,8 @@
 /*******************************************************************************
 *                                                                              *
 * Author    :  Angus Johnson                                                   *
-* Version   :  4.3.0                                                           *
-* Date      :  16 June 2011                                                    *
+* Version   :  4.4.4                                                           *
+* Date      :  4 September 2011                                                *
 * Website   :  http://www.angusj.com                                           *
 * Copyright :  Angus Johnson 2010-2011                                         *
 *                                                                              *
@@ -30,34 +30,50 @@
 #include <stdexcept>
 #include <cstring>
 #include <cstdlib>
+#include <ostream>
 
-namespace clipper {
+namespace ClipperLib {
 
 enum ClipType { ctIntersection, ctUnion, ctDifference, ctXor };
 enum PolyType { ptSubject, ptClip };
+//By far the most widely used winding rules for polygon filling are
+//EvenOdd & NonZero (GDI, GDI+, XLib, OpenGL, Cairo, AGG, Quartz, SVG, Gr32)
+//Others rules include Positive, Negative and ABS_GTR_EQ_TWO (only in OpenGL)
+//see http://www.songho.ca/opengl/gl_tessellation.html#winding_rules
 enum PolyFillType { pftEvenOdd, pftNonZero };
 
 typedef signed long long long64;
 typedef unsigned long long ulong64;
 
 struct IntPoint {
+public:
   long64 X;
   long64 Y;
   IntPoint(long64 x = 0, long64 y = 0): X(x), Y(y) {};
+  friend std::ostream& operator <<(std::ostream &s, IntPoint &p);
 };
 
 typedef std::vector< IntPoint > Polygon;
 typedef std::vector< Polygon > Polygons;
 
+std::ostream& operator <<(std::ostream &s, Polygon &p);
+std::ostream& operator <<(std::ostream &s, Polygons &p);
+
 struct ExPolygon {
   Polygon  outer;
   Polygons holes;
 };
 typedef std::vector< ExPolygon > ExPolygons;
 
+enum JoinType { jtSquare, jtMiter, jtRound };
+
 bool IsClockwise(const Polygon &poly, bool UseFullInt64Range = true);
 double Area(const Polygon &poly, bool UseFullInt64Range = true);
-bool OffsetPolygons(const Polygons &in_pgs, Polygons &out_pgs, const float &delta);
+void OffsetPolygons(const Polygons &in_polys, Polygons &out_polys,
+  double delta, JoinType jointype, double MiterLimit = 2);
+
+void ReversePoints(Polygon& p);
+void ReversePoints(Polygons& p);
 
 //used internally ...
 enum EdgeSide { esLeft, esRight };
@@ -115,6 +131,8 @@ struct OutRec {
   OutRec *AppendLink;
   OutPt  *pts;
   OutPt  *bottomPt;
+  TEdge  *bottomE1;
+  TEdge  *bottomE2;
 };
 
 struct OutPt {
@@ -157,7 +175,7 @@ public:
   bool AddPolygons( const Polygons &ppg, PolyType polyType);
   virtual void Clear();
   IntRect GetBounds();
-  bool UseFullCoordinateRange() {return m_UseFullRange;};
+  bool UseFullCoordinateRange() {return m_UseFullRange;}; //default = false
   void UseFullCoordinateRange(bool newVal);
 protected:
   void DisposeLocalMinimaList();
@@ -228,12 +246,13 @@ private:
   void DoBothEdges(TEdge *edge1, TEdge *edge2, const IntPoint &pt);
   void IntersectEdges(TEdge *e1, TEdge *e2,
     const IntPoint &pt, IntersectProtects protects);
-  void AddOutPt(TEdge *e, const IntPoint &pt);
+  OutRec* CreateOutRec();
+  void AddOutPt(TEdge *e, TEdge *altE, const IntPoint &pt);
   void DisposeAllPolyPts();
   void DisposeOutRec(int index, bool ignorePts = false);
-  bool ProcessIntersections( const long64 topY);
+  bool ProcessIntersections(const long64 botY, const long64 topY);
   void AddIntersectNode(TEdge *e1, TEdge *e2, const IntPoint &pt);
-  void BuildIntersectList(const long64 topY);
+  void BuildIntersectList(const long64 botY, const long64 topY);
   void ProcessIntersectList();
   void ProcessEdgesAtTopOfScanbeam(const long64 topY);
   void BuildResult(Polygons& polys);
@@ -265,9 +284,9 @@ class clipperException : public std::exception
     std::string m_description;
 };
 //------------------------------------------------------------------------------
-//------------------------------------------------------------------------------
 
-} //clipper namespace
+} //ClipperLib namespace
+
 #endif //clipper_hpp
 
 
